@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2018 Junjiro R. Okajima
+ * Copyright (C) 2005-2019 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -234,7 +234,7 @@ int au_lkup_dentry(struct dentry *dentry, aufs_bindex_t btop,
 
 out_parent:
 	dput(parent);
-	kfree(args.whname.name);
+	au_kfree_try_rcu(args.whname.name);
 	if (dirren)
 		au_dr_lkup_fin(&args);
 out:
@@ -608,7 +608,7 @@ static int au_refresh_by_dinfo(struct dentry *dentry, struct au_dinfo *dinfo,
 	if (d_really_is_positive(dentry))
 		inode = d_inode(dentry);
 	if (!orig_h.inode) {
-		AuDbg("nagative originally\n");
+		AuDbg("negative originally\n");
 		if (inode) {
 			au_hide(dentry);
 			goto out;
