@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2018 Junjiro R. Okajima
+ * Copyright (C) 2005-2019 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -157,7 +157,7 @@ out_unlock:
 out:
 	dput(a->dentry);
 	au_nwt_done(&au_sbi(sb)->si_nowait);
-	kfree(arg);
+	au_kfree_try_rcu(arg);
 }
 
 void au_dir_ts(struct inode *dir, aufs_bindex_t bindex)
@@ -193,7 +193,7 @@ void au_dir_ts(struct inode *dir, aufs_bindex_t bindex)
 	if (unlikely(wkq_err)) {
 		pr_err("wkq %d\n", wkq_err);
 		dput(dentry);
-		kfree(arg);
+		au_kfree_try_rcu(arg);
 	}
 
 out:
@@ -312,7 +312,7 @@ static int aufs_open_dir(struct inode *inode __maybe_unused,
 		};
 		err = au_do_open(file, &args);
 		if (unlikely(err))
-			kfree(fidir);
+			au_kfree_rcu(fidir);
 	}
 	si_read_unlock(sb);
 	return err;
@@ -348,7 +348,7 @@ static int aufs_release_dir(struct inode *inode __maybe_unused,
 				if (hf->hf_file)
 					au_hfput(hf, /*execed*/0);
 		}
-		kfree(fidir);
+		au_kfree_rcu(fidir);
 		finfo->fi_hdir = NULL;
 	}
 	au_finfo_fin(file);
@@ -588,6 +588,7 @@ static int do_test_empty(struct dentry *dentry, struct test_empty_arg *arg)
 {
 	int err;
 	struct file *h_file;
+	struct au_branch *br;
 
 	h_file = au_h_open(dentry, arg->bindex,
 			   O_RDONLY | O_NONBLOCK | O_DIRECTORY | O_LARGEFILE,
@@ -612,7 +613,8 @@ static int do_test_empty(struct dentry *dentry, struct test_empty_arg *arg)
 
 out_put:
 	fput(h_file);
-	au_sbr_put(dentry->d_sb, arg->bindex);
+	br = au_sbr(dentry->d_sb, arg->bindex);
+	au_lcnt_dec(&br->br_nfiles);
 out:
 	return err;
 }
