@@ -1,5 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2017 Junjiro R. Okajima
+ * Copyright (C) 2017-2019 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +17,7 @@
  */
 
 /*
- * special handling in renaming a directoy
+ * special handling in renaming a directory
  * in order to support looking-up the before-renamed name on the lower readonly
  * branches
  */
@@ -117,7 +118,7 @@ void au_dr_hino_free(struct au_dr_br *dr)
 		hbl = dr->dr_h_ino + i;
 		/* no spinlock since sbinfo must be write-locked */
 		hlist_bl_for_each_entry_safe(ent, pos, tmp, hbl, dr_hnode)
-			kfree(ent);
+			au_kfree_rcu(ent);
 		INIT_HLIST_BL_HEAD(hbl);
 	}
 }
@@ -559,7 +560,7 @@ static struct au_drinfo *au_drinfo_read_k(struct file *file, ino_t h_ino)
 	*ret = *drinfo;
 	ssz = vfsub_read_k(file, (void *)ret->oldname, len, &pos);
 	if (unlikely(ssz != len)) {
-		kfree(ret);
+		au_kfree_rcu(ret);
 		ret = ERR_PTR(-EIO);
 		AuIOErr("ssz %zd, %u, %pD2\n", ssz, len, file);
 		goto out;
@@ -597,7 +598,7 @@ struct au_drinfo_store {
 	unsigned char no_sio,
 		allocated,		/* current size of *fdata */
 		infonamelen,		/* room size for p */
-		whnamelen,		/* length of the genarated name */
+		whnamelen,		/* length of the generated name */
 		renameback;		/* renamed back */
 };
 
@@ -755,7 +756,7 @@ out:
 
 static void au_drinfo_store_work_fin(struct au_drinfo_store *w)
 {
-	kfree(w->fdata);
+	au_kfree_rcu(w->fdata);
 }
 
 static void au_drinfo_store_rev(struct au_drinfo_rev *rev,
@@ -792,7 +793,7 @@ static void au_drinfo_store_rev(struct au_drinfo_rev *rev,
 			       elm->info_last->oldname,
 			       elm->info_last->oldnamelen);
 			err = au_drinfo_store_sio(w, /*elm*/NULL);
-			kfree(elm->info_last);
+			au_kfree_rcu(elm->info_last);
 		}
 		if (unlikely(err))
 			AuIOErr("%d, %s\n", err, w->whname);
@@ -868,7 +869,7 @@ static int au_drinfo_store(struct dentry *dentry, aufs_bindex_t btgt,
 	if (unlikely(err)) {
 		/* revert all drinfo */
 		au_drinfo_store_rev(rev, &work);
-		kfree(rev);
+		au_kfree_try_rcu(rev);
 		*p = NULL;
 	}
 	au_hn_inode_unlock(hdir);
@@ -924,7 +925,7 @@ int au_dr_rename(struct dentry *src, aufs_bindex_t bindex,
 	/* revert */
 	if (!already)
 		au_dr_hino_del(dr, ent);
-	kfree(ent);
+	au_kfree_rcu(ent);
 
 out:
 	AuTraceErr(err);
@@ -941,9 +942,9 @@ void au_dr_rename_fin(struct dentry *src, aufs_bindex_t btgt, void *_rev)
 	elm = rev->elm;
 	for (nelm = rev->nelm; nelm > 0; nelm--, elm++) {
 		dput(elm->info_dentry);
-		kfree(elm->info_last);
+		au_kfree_rcu(elm->info_last);
 	}
-	kfree(rev);
+	au_kfree_try_rcu(rev);
 }
 
 void au_dr_rename_rev(struct dentry *src, aufs_bindex_t btgt, void *_rev)
@@ -975,10 +976,10 @@ void au_dr_rename_rev(struct dentry *src, aufs_bindex_t btgt, void *_rev)
 	ent = au_dr_hino_find(dr, h_inode->i_ino);
 	BUG_ON(!ent);
 	au_dr_hino_del(dr, ent);
-	kfree(ent);
+	au_kfree_rcu(ent);
 
 out:
-	kfree(rev);
+	au_kfree_try_rcu(rev);
 	if (unlikely(err))
 		pr_err("failed to remove dirren info\n");
 }
@@ -1001,7 +1002,7 @@ static struct au_drinfo *au_drinfo_do_load(struct path *h_ppath,
 	drinfo = NULL;
 	unlocked = 0;
 	h_dir = d_inode(h_ppath->dentry);
-	vfsub_inode_lock_shared_nested(h_dir, AuLsc_I_PARENT);
+	inode_lock_shared_nested(h_dir, AuLsc_I_PARENT);
 	infopath.dentry = vfsub_lookup_one_len(whname, h_ppath->dentry,
 					       whnamelen);
 	if (IS_ERR(infopath.dentry)) {
@@ -1103,7 +1104,7 @@ static int au_drinfo_load(struct au_drinfo_load *w, aufs_bindex_t bindex,
 	oldname.name = drinfo->oldname;
 	if (au_qstreq(w->qname, &oldname)) {
 		/* the name is renamed back */
-		kfree(drinfo);
+		au_kfree_rcu(drinfo);
 		drinfo = NULL;
 
 		infopath.dentry = info_dentry;
@@ -1118,7 +1119,7 @@ static int au_drinfo_load(struct au_drinfo_load *w, aufs_bindex_t bindex,
 		if (unlikely(e == -EWOULDBLOCK))
 			iput(delegated);
 	}
-	kfree(w->drinfo[bindex]);
+	au_kfree_rcu(w->drinfo[bindex]);
 	w->drinfo[bindex] = drinfo;
 	dput(info_dentry);
 
@@ -1134,8 +1135,8 @@ static void au_dr_lkup_free(struct au_drinfo **drinfo, int n)
 	struct au_drinfo **p = drinfo;
 
 	while (n-- > 0)
-		kfree(*drinfo++);
-	kfree(p);
+		au_kfree_rcu(*drinfo++);
+	au_kfree_try_rcu(p);
 }
 
 int au_dr_lkup(struct au_do_lookup_args *lkup, struct dentry *dentry,
@@ -1188,7 +1189,7 @@ int au_dr_lkup(struct au_do_lookup_args *lkup, struct dentry *dentry,
 		ent = au_dr_hino_find(&br->br_dirren, h_dir->i_ino);
 		AuDebugOn(!ent);
 		au_dr_hino_del(&br->br_dirren, ent);
-		kfree(ent);
+		au_kfree_rcu(ent);
 	}
 	goto out; /* success */
 
@@ -1219,7 +1220,7 @@ int au_dr_lkup_name(struct au_do_lookup_args *lkup, aufs_bindex_t btgt)
 	if (!drinfo)
 		goto out;
 
-	kfree(lkup->whname.name);
+	au_kfree_try_rcu(lkup->whname.name);
 	lkup->whname.name = NULL;
 	lkup->dirren.dr_name.len = drinfo->oldnamelen;
 	lkup->dirren.dr_name.name = drinfo->oldname;
