@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2018 Junjiro R. Okajima
+ * Copyright (C) 2005-2019 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -62,8 +62,7 @@ static int au_hfsn_alloc(struct au_hinode *hinode)
 	 * h_inode, so specify 1 to allow dups.
 	 */
 	lockdep_off();
-	err = fsnotify_add_mark(mark, hinode->hi_inode, /*mnt*/NULL,
-				/*allow_dups*/1);
+	err = fsnotify_add_inode_mark(mark, hinode->hi_inode, /*allow_dups*/1);
 	lockdep_on();
 
 	return err;
@@ -157,7 +156,7 @@ static void au_hfsn_free_group(struct fsnotify_group *group)
 	struct au_br_hfsnotify *hfsn = group->private;
 
 	/* AuDbg("here\n"); */
-	kfree(hfsn);
+	au_kfree_try_rcu(hfsn);
 }
 
 static int au_hfsn_handle_event(struct fsnotify_group *group,
@@ -253,7 +252,7 @@ static int au_hfsn_init_br(struct au_branch *br, int perm)
 	goto out; /* success */
 
 out_hfsn:
-	kfree(hfsn);
+	au_kfree_try_rcu(hfsn);
 out:
 	return err;
 }
