@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2018 Junjiro R. Okajima
+ * Copyright (C) 2005-2019 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -252,6 +252,28 @@ out:
 
 /* ---------------------------------------------------------------------- */
 
+/*
+ * very dirty and complicated aufs ->atomic_open().
+ * aufs_atomic_open()
+ * + au_aopen_or_create()
+ *   + add_simple()
+ *     + vfsub_atomic_open()
+ *       + branch fs ->atomic_open()
+ *	   may call the actual 'open' for h_file
+ *       + inc br_nfiles only if opened
+ * + au_aopen_no_open() or au_aopen_do_open()
+ *
+ * au_aopen_do_open()
+ * + finish_open()
+ *   + au_do_aopen()
+ *     + au_do_open() the body of all 'open'
+ *       + au_do_open_nondir()
+ *	   set the passed h_file
+ *
+ * au_aopen_no_open()
+ * + finish_no_open()
+ */
+
 struct aopen_node {
 	struct hlist_bl_node hblist;
 	struct file *file, *h_file;
@@ -280,18 +302,50 @@ static int au_do_aopen(struct inode *inode, struct file *file)
 	return au_do_open(file, &args);
 }
 
+static int au_aopen_do_open(struct file *file, struct dentry *dentry,
+			    struct aopen_node *aopen_node)
+{
+	int err;
+	struct hlist_bl_head *aopen;
+
+	AuLabel(here);
+	aopen = &au_sbi(dentry->d_sb)->si_aopen;
+	au_hbl_add(&aopen_node->hblist, aopen);
+	err = finish_open(file, dentry, au_do_aopen);
+	au_hbl_del(&aopen_node->hblist, aopen);
+	/* AuDbgFile(file); */
+	AuDbg("%pd%s%s\n", dentry,
+	      (file->f_mode & FMODE_CREATED) ? " created" : "",
+	      (file->f_mode & FMODE_OPENED) ? " opened" : "");
+
+	AuTraceErr(err);
+	return err;
+}
+
+static int au_aopen_no_open(struct file *file, struct dentry *dentry)
+{
+	int err;
+
+	AuLabel(here);
+	dget(dentry);
+	err = finish_no_open(file, dentry);
+
+	AuTraceErr(err);
+	return err;
+}
+
 static int aufs_atomic_open(struct inode *dir, struct dentry *dentry,
 			    struct file *file, unsigned int open_flag,
-			    umode_t create_mode, int *opened)
+			    umode_t create_mode)
 {
-	int err, unlocked, h_opened = *opened;
+	int err, did_open;
 	unsigned int lkup_flags;
+	aufs_bindex_t bindex;
+	struct super_block *sb;
 	struct dentry *parent, *d;
-	struct hlist_bl_head *aopen;
 	struct vfsub_aopen_args args = {
 		.open_flag	= open_flag,
-		.create_mode	= create_mode,
-		.opened		= &h_opened
+		.create_mode	= create_mode
 	};
 	struct aopen_node aopen_node = {
 		.file	= file
@@ -326,82 +380,73 @@ static int aufs_atomic_open(struct inode *dir, struct dentry *dentry,
 	if (d_is_positive(dentry)
 	    || d_unhashed(dentry)
 	    || d_unlinked(dentry)
-	    || !(open_flag & O_CREAT))
-		goto out_no_open;
+	    || !(open_flag & O_CREAT)) {
+		err = au_aopen_no_open(file, dentry);
+		goto out; /* success */
+	}
 
-	unlocked = 0;
 	err = aufs_read_lock(dentry, AuLock_DW | AuLock_FLUSH | AuLock_GEN);
 	if (unlikely(err))
 		goto out;
 
+	sb = dentry->d_sb;
 	parent = dentry->d_parent;	/* dir is locked */
 	di_write_lock_parent(parent);
 	err = au_lkup_dentry(dentry, /*btop*/0, AuLkup_ALLOW_NEG);
-	if (unlikely(err))
-		goto out_unlock;
+	if (unlikely(err < 0))
+		goto out_parent;
 
 	AuDbgDentry(dentry);
-	if (d_is_positive(dentry))
-		goto out_unlock;
+	if (d_is_positive(dentry)) {
+		err = au_aopen_no_open(file, dentry);
+		goto out_parent; /* success */
+	}
 
-	args.file = get_empty_filp();
+	args.file = alloc_empty_file(file->f_flags, current_cred());
 	err = PTR_ERR(args.file);
 	if (IS_ERR(args.file))
-		goto out_unlock;
+		goto out_parent;
 
-	args.file->f_flags = file->f_flags;
+	bindex = au_dbtop(dentry);
 	err = au_aopen_or_create(dir, dentry, &args);
 	AuTraceErr(err);
 	AuDbgFile(args.file);
-	if (unlikely(err < 0)) {
-		if (h_opened & FILE_OPENED)
-			fput(args.file);
-		else
-			put_filp(args.file);
-		goto out_unlock;
+	file->f_mode = args.file->f_mode & ~FMODE_OPENED;
+	did_open = !!(args.file->f_mode & FMODE_OPENED);
+	if (!did_open) {
+		fput(args.file);
+		args.file = NULL;
 	}
 	di_write_unlock(parent);
 	di_write_unlock(dentry);
-	unlocked = 1;
-
-	/* some filesystems don't set FILE_CREATED while succeeded? */
-	*opened |= FILE_CREATED;
-	if (h_opened & FILE_OPENED)
-		aopen_node.h_file = args.file;
-	else {
-		put_filp(args.file);
-		args.file = NULL;
+	if (unlikely(err < 0)) {
+		if (args.file)
+			fput(args.file);
+		goto out_sb;
 	}
-	aopen = &au_sbi(dir->i_sb)->si_aopen;
-	au_hbl_add(&aopen_node.hblist, aopen);
-	err = finish_open(file, dentry, au_do_aopen, opened);
-	au_hbl_del(&aopen_node.hblist, aopen);
+
+	if (!did_open)
+		err = au_aopen_no_open(file, dentry);
+	else {
+		aopen_node.h_file = args.file;
+		err = au_aopen_do_open(file, dentry, &aopen_node);
+	}
+	if (unlikely(err < 0)) {
+		if (args.file)
+			fput(args.file);
+		if (did_open)
+			au_lcnt_dec(&args.br->br_nfiles);
+	}
+	goto out_sb; /* success */
+
+out_parent:
+	di_write_unlock(parent);
+	di_write_unlock(dentry);
+out_sb:
+	si_read_unlock(sb);
+out:
 	AuTraceErr(err);
 	AuDbgFile(file);
-	if (aopen_node.h_file)
-		fput(aopen_node.h_file);
-
-out_unlock:
-	if (unlocked)
-		si_read_unlock(dentry->d_sb);
-	else {
-		di_write_unlock(parent);
-		aufs_read_unlock(dentry, AuLock_DW);
-	}
-	AuDbgDentry(dentry);
-	if (unlikely(err < 0))
-		goto out;
-out_no_open:
-	if (err >= 0 && !(*opened & FILE_CREATED)) {
-		AuLabel(out_no_open);
-		dget(dentry);
-		err = finish_no_open(file, dentry);
-	}
-out:
-	AuDbg("%pd%s%s\n", dentry,
-	      (*opened & FILE_CREATED) ? " created" : "",
-	      (*opened & FILE_OPENED) ? " opened" : "");
-	AuTraceErr(err);
 	return err;
 }
 
@@ -1011,7 +1056,7 @@ out_dentry:
 out_si:
 	si_read_unlock(sb);
 out_kfree:
-	kfree(a);
+	au_kfree_rcu(a);
 out:
 	AuTraceErr(err);
 	return err;
@@ -1102,7 +1147,7 @@ out_di:
 	di_write_unlock(dentry);
 	si_read_unlock(sb);
 out_kfree:
-	kfree(a);
+	au_kfree_rcu(a);
 out:
 	AuTraceErr(err);
 	return err;
@@ -1335,7 +1380,8 @@ static int au_is_special(struct inode *inode)
 	return (inode->i_mode & (S_IFBLK | S_IFCHR | S_IFIFO | S_IFSOCK));
 }
 
-static int aufs_update_time(struct inode *inode, struct timespec *ts, int flags)
+static int aufs_update_time(struct inode *inode, struct timespec64 *ts,
+			    int flags)
 {
 	int err;
 	aufs_bindex_t bindex;
