@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2018 Junjiro R. Okajima
+ * Copyright (C) 2005-2019 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -659,7 +659,7 @@ out:
  * locking order
  * (VFS)
  * - src_dir and dir by lock_rename()
- * - inode if exitsts
+ * - inode if exists
  * (aufs)
  * - lock all
  *   + src_dentry and dentry by aufs_read_and_write_lock2() which calls,
@@ -977,6 +977,8 @@ int aufs_rename(struct inode *_src_dir, struct dentry *_src_dentry,
 		goto out;
 
 	a->flags = _flags;
+	BUILD_BUG_ON(sizeof(a->exchange) == sizeof(u8)
+		     && RENAME_EXCHANGE > U8_MAX);
 	a->exchange = _flags & RENAME_EXCHANGE;
 	a->src_dir = _src_dir;
 	a->src_dentry = _src_dentry;
@@ -1240,7 +1242,7 @@ out_free:
 	iput(a->dst_inode);
 	if (a->thargs)
 		au_whtmp_rmdir_free(a->thargs);
-	kfree(a);
+	au_kfree_rcu(a);
 out:
 	AuTraceErr(err);
 	return err;
