@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2018 Junjiro R. Okajima
+ * Copyright (C) 2005-2019 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -865,7 +865,7 @@ au_opts_parse_mod(struct au_opt_mod *mod, substring_t args[])
 	mod->path = args[0].from;
 	p = strchr(mod->path, '=');
 	if (unlikely(!p)) {
-		pr_err("no permssion %s\n", args[0].from);
+		pr_err("no permission %s\n", args[0].from);
 		goto out;
 	}
 
@@ -1260,7 +1260,7 @@ int au_opts_parse(struct super_block *sb, char *str, struct au_opts *opts)
 		}
 	}
 
-	kfree(a);
+	au_kfree_rcu(a);
 	dump_opts(opts);
 	if (unlikely(err))
 		au_opts_free(opts);
@@ -1554,25 +1554,10 @@ static int au_opt_xino(struct super_block *sb, struct au_opt *opt,
 		       struct au_opts *opts)
 {
 	int err;
-	aufs_bindex_t bbot, bindex;
-	struct dentry *root, *parent, *h_root;
 
 	err = 0;
 	switch (opt->type) {
 	case Opt_xino:
-		au_xino_brid_set(sb, -1);
-		/* safe d_parent access */
-		parent = opt->xino.file->f_path.dentry->d_parent;
-		root = sb->s_root;
-		bbot = au_sbbot(sb);
-		for (bindex = 0; bindex <= bbot; bindex++) {
-			h_root = au_h_dptr(root, bindex);
-			if (h_root == parent) {
-				au_xino_brid_set(sb, au_sbr_id(sb, bindex));
-				break;
-			}
-		}
-
 		err = au_xino_set(sb, &opt->xino,
 				  !!au_ftest_opts(opts->flags, REMOUNT));
 		if (unlikely(err))
@@ -1705,7 +1690,7 @@ int au_opts_verify(struct super_block *sb, unsigned long sb_flags,
 		au_hn_inode_unlock(hdir);
 
 		if (!err && do_free) {
-			kfree(wbr);
+			au_kfree_rcu(wbr);
 			br->br_wbr = NULL;
 		}
 	}
