@@ -1,6 +1,6 @@
 /* SPDX-License-Identifier: GPL-2.0 */
 /*
- * Copyright (C) 2005-2018 Junjiro R. Okajima
+ * Copyright (C) 2005-2019 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include <linux/fs.h>
 #include <linux/kobject.h>
 #include "hbl.h"
+#include "lcnt.h"
 #include "rwsem.h"
 #include "wkq.h"
 
@@ -86,7 +87,7 @@ struct au_sbinfo {
 	struct au_nowait_tasks	si_nowait;
 
 	/*
-	 * tried sb->s_umount, but failed due to the dependecy between i_mutex.
+	 * tried sb->s_umount, but failed due to the dependency between i_mutex.
 	 * rwsem for au_sbinfo is necessary.
 	 */
 	struct au_rwsem		si_rwsem;
@@ -95,7 +96,7 @@ struct au_sbinfo {
 	 * dirty approach to protect sb->sb_inodes and ->s_files (gone) from
 	 * remount.
 	 */
-	struct percpu_counter	si_ninodes, si_nfiles;
+	au_lcnt_t		si_ninodes, si_nfiles;
 
 	/* branch management */
 	unsigned int		si_generation;
@@ -140,7 +141,6 @@ struct au_sbinfo {
 	unsigned long		si_xib_last_pindex;
 	int			si_xib_next_bit;
 
-	aufs_bindex_t		si_xino_brid;
 	unsigned long		si_xino_jiffy;
 	unsigned long		si_xino_expire;
 	/* reserved for future use */
@@ -153,7 +153,7 @@ struct au_sbinfo {
 	atomic_t		si_xigen_next;
 #endif
 
-	/* dirty trick to suppoer atomic_open */
+	/* dirty trick to support atomic_open */
 	struct hlist_bl_head	si_aopen;
 
 	/* vdir parameters */
@@ -184,7 +184,7 @@ struct au_sbinfo {
 	/*
 	 * sysfs and lifetime management.
 	 * this is not a small structure and it may be a waste of memory in case
-	 * of sysfs is disabled, particulary when many aufs-es are mounted.
+	 * of sysfs is disabled, particularly when many aufs-es are mounted.
 	 * but using sysfs is majority.
 	 */
 	struct kobject		si_kobj;
@@ -572,42 +572,6 @@ static inline unsigned int au_sigen(struct super_block *sb)
 	return au_sbi(sb)->si_generation;
 }
 
-static inline unsigned long long au_ninodes(struct super_block *sb)
-{
-	s64 n = percpu_counter_sum(&au_sbi(sb)->si_ninodes);
-
-	BUG_ON(n < 0);
-	return n;
-}
-
-static inline void au_ninodes_inc(struct super_block *sb)
-{
-	percpu_counter_inc(&au_sbi(sb)->si_ninodes);
-}
-
-static inline void au_ninodes_dec(struct super_block *sb)
-{
-	percpu_counter_dec(&au_sbi(sb)->si_ninodes);
-}
-
-static inline unsigned long long au_nfiles(struct super_block *sb)
-{
-	s64 n = percpu_counter_sum(&au_sbi(sb)->si_nfiles);
-
-	BUG_ON(n < 0);
-	return n;
-}
-
-static inline void au_nfiles_inc(struct super_block *sb)
-{
-	percpu_counter_inc(&au_sbi(sb)->si_nfiles);
-}
-
-static inline void au_nfiles_dec(struct super_block *sb)
-{
-	percpu_counter_dec(&au_sbi(sb)->si_nfiles);
-}
-
 static inline struct au_branch *au_sbr(struct super_block *sb,
 				       aufs_bindex_t bindex)
 {
@@ -615,18 +579,6 @@ static inline struct au_branch *au_sbr(struct super_block *sb,
 	return au_sbi(sb)->si_branch[0 + bindex];
 }
 
-static inline void au_xino_brid_set(struct super_block *sb, aufs_bindex_t brid)
-{
-	SiMustWriteLock(sb);
-	au_sbi(sb)->si_xino_brid = brid;
-}
-
-static inline aufs_bindex_t au_xino_brid(struct super_block *sb)
-{
-	SiMustAnyLock(sb);
-	return au_sbi(sb)->si_xino_brid;
-}
-
 static inline loff_t au_xi_maxent(struct super_block *sb)
 {
 	SiMustAnyLock(sb);
