@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2018 Junjiro R. Okajima
+ * Copyright (C) 2005-2019 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,6 +73,28 @@ static aufs_bindex_t is_sb_shared(struct super_block *sb, aufs_bindex_t btgt,
 /*
  * stop unnecessary notify events at creating xino files
  */
+
+aufs_bindex_t au_xi_root(struct super_block *sb, struct dentry *dentry)
+{
+	aufs_bindex_t bfound, bindex, bbot;
+	struct dentry *parent;
+	struct au_branch *br;
+
+	bfound = -1;
+	parent = dentry->d_parent; /* safe d_parent access */
+	bbot = au_sbbot(sb);
+	for (bindex = 0; bindex <= bbot; bindex++) {
+		br = au_sbr(sb, bindex);
+		if (au_br_dentry(br) == parent) {
+			bfound = bindex;
+			break;
+		}
+	}
+
+	AuDbg("bfound b%d\n", bfound);
+	return bfound;
+}
+
 struct au_xino_lock_dir {
 	struct au_hinode *hdir;
 	struct dentry *parent;
@@ -107,13 +129,10 @@ out:
 static void au_xino_lock_dir(struct super_block *sb, struct path *xipath,
 			     struct au_xino_lock_dir *ldir)
 {
-	aufs_bindex_t brid, bindex;
+	aufs_bindex_t bindex;
 
 	ldir->hdir = NULL;
-	bindex = -1;
-	brid = au_xino_brid(sb);
-	if (brid >= 0)
-		bindex = au_br_index(sb, brid);
+	bindex = au_xi_root(sb, xipath->dentry);
 	if (bindex >= 0) {
 		/* rw branch root */
 		ldir->hdir = au_hi(d_inode(sb->s_root), bindex);
@@ -453,7 +472,7 @@ int au_xino_trunc(struct super_block *sb, aufs_bindex_t bindex, int idx_begin)
 		au_sbi(sb)->si_xino_jiffy = jiffy;
 
 out_st:
-	kfree(st);
+	au_kfree_rcu(st);
 out:
 	return err;
 }
@@ -487,14 +506,14 @@ static void xino_do_trunc(void *_args)
 	if (unlikely(err))
 		pr_warn("err b%d, (%d)\n", bindex, err);
 	atomic_dec(&br->br_xino->xi_truncating);
-	au_br_put(br);
+	au_lcnt_dec(&br->br_count);
 	si_write_unlock(sb);
 	au_nwt_done(&au_sbi(sb)->si_nowait);
-	kfree(args);
+	au_kfree_rcu(args);
 }
 
 /*
- * returs the index in the xi_file array whose corresponding file is necessary
+ * returns the index in the xi_file array whose corresponding file is necessary
  * to truncate, or -1 which means no need to truncate.
  */
 static int xino_trunc_test(struct super_block *sb, struct au_branch *br)
@@ -552,7 +571,7 @@ static void xino_try_trunc(struct super_block *sb, struct au_branch *br)
 		goto out;
 	}
 
-	au_br_get(br);
+	au_lcnt_inc(&br->br_count);
 	args->sb = sb;
 	args->br = br;
 	args->idx = idx;
@@ -561,8 +580,8 @@ static void xino_try_trunc(struct super_block *sb, struct au_branch *br)
 		return; /* success */
 
 	pr_err("wkq %d\n", wkq_err);
-	au_br_put(br);
-	kfree(args);
+	au_lcnt_dec(&br->br_count);
+	au_kfree_rcu(args);
 
 out:
 	atomic_dec(&br->br_xino->xi_truncating);
@@ -628,6 +647,11 @@ struct au_xino_do_new_async_args {
 	ino_t ino;
 };
 
+struct au_xi_writing {
+	struct hlist_bl_node node;
+	ino_t h_ino, ino;
+};
+
 static int au_xino_do_write(vfs_writef_t write, struct file *file,
 			    struct au_xi_calc *calc, ino_t ino);
 
@@ -639,6 +663,9 @@ static void au_xino_call_do_new_async(void *args)
 	struct au_sbinfo *sbi;
 	struct inode *root;
 	struct file *file;
+	struct au_xi_writing *del, *p;
+	struct hlist_bl_head *hbl;
+	struct hlist_bl_node *pos;
 	int err;
 
 	br = a->br;
@@ -648,24 +675,43 @@ static void au_xino_call_do_new_async(void *args)
 	root = d_inode(sb->s_root);
 	ii_read_lock_child(root);
 	err = au_xino_do_new_async(sb, br, &a->calc);
-	if (!err) {
-		file = au_xino_file(br->br_xino, a->calc.idx);
-		if (file)
-			err = au_xino_do_write(sbi->si_xwrite, file, &a->calc,
-					       a->ino);
-		if (unlikely(err))
-			AuIOErr("err %d\n", err);
-	} else
+	if (unlikely(err)) {
 		AuIOErr("err %d\n", err);
-	au_br_put(br);
+		goto out;
+	}
+
+	file = au_xino_file(br->br_xino, a->calc.idx);
+	AuDebugOn(!file);
+	err = au_xino_do_write(sbi->si_xwrite, file, &a->calc, a->ino);
+	if (unlikely(err)) {
+		AuIOErr("err %d\n", err);
+		goto out;
+	}
+
+	del = NULL;
+	hbl = &br->br_xino->xi_writing;
+	hlist_bl_lock(hbl);
+	au_hbl_for_each(pos, hbl) {
+		p = container_of(pos, struct au_xi_writing, node);
+		if (p->ino == a->ino) {
+			del = p;
+			hlist_bl_del(&p->node);
+			break;
+		}
+	}
+	hlist_bl_unlock(hbl);
+	au_kfree_rcu(del);
+
+out:
+	au_lcnt_dec(&br->br_count);
 	ii_read_unlock(root);
 	si_read_unlock(sb);
 	au_nwt_done(&sbi->si_nowait);
-	kfree(args);
+	au_kfree_rcu(a);
 }
 
 /*
- * create a new xino file asynchoronously
+ * create a new xino file asynchronously
  */
 static int au_xino_new_async(struct super_block *sb, struct au_branch *br,
 			     struct au_xi_calc *calc, ino_t ino)
@@ -682,12 +728,12 @@ static int au_xino_new_async(struct super_block *sb, struct au_branch *br,
 	arg->br = br;
 	arg->calc = *calc;
 	arg->ino = ino;
-	au_br_get(br);
+	au_lcnt_inc(&br->br_count);
 	err = au_wkq_nowait(au_xino_call_do_new_async, arg, sb, AuWkq_NEST);
 	if (unlikely(err)) {
 		pr_err("wkq %d\n", err);
-		au_br_put(br);
-		kfree(arg);
+		au_lcnt_dec(&br->br_count);
+		au_kfree_rcu(arg);
 	}
 
 out:
@@ -706,6 +752,10 @@ int au_xino_read(struct super_block *sb, aufs_bindex_t bindex, ino_t h_ino,
 	struct au_xi_calc calc;
 	struct au_sbinfo *sbinfo;
 	struct file *file;
+	struct au_xino *xi;
+	struct hlist_bl_head *hbl;
+	struct hlist_bl_node *pos;
+	struct au_xi_writing *p;
 
 	*ino = 0;
 	if (!au_opt_test(au_mntflags(sb), XINO))
@@ -713,10 +763,24 @@ int au_xino_read(struct super_block *sb, aufs_bindex_t bindex, ino_t h_ino,
 
 	err = 0;
 	au_xi_calc(sb, h_ino, &calc);
-	file = au_xino_file(au_sbr(sb, bindex)->br_xino, calc.idx);
-	if (!file
-	    || vfsub_f_size_read(file) < calc.pos + sizeof(*ino))
-		return 0; /* no ino */
+	xi = au_sbr(sb, bindex)->br_xino;
+	file = au_xino_file(xi, calc.idx);
+	if (!file) {
+		hbl = &xi->xi_writing;
+		hlist_bl_lock(hbl);
+		au_hbl_for_each(pos, hbl) {
+			p = container_of(pos, struct au_xi_writing, node);
+			if (p->h_ino == h_ino) {
+				AuDbg("hi%llu, i%llu, found\n",
+				      (u64)p->h_ino, (u64)p->ino);
+				*ino = p->ino;
+				break;
+			}
+		}
+		hlist_bl_unlock(hbl);
+		return 0;
+	} else if (vfsub_f_size_read(file) < calc.pos + sizeof(*ino))
+		return 0; /* no xino */
 
 	sbinfo = au_sbi(sb);
 	sz = xino_fread(sbinfo->si_xread, file, ino, sizeof(*ino), &calc.pos);
@@ -760,6 +824,7 @@ int au_xino_write(struct super_block *sb, aufs_bindex_t bindex, ino_t h_ino,
 	struct file *file;
 	struct au_branch *br;
 	struct au_xino *xi;
+	struct au_xi_writing *p;
 
 	SiMustAnyLock(sb);
 
@@ -772,7 +837,13 @@ int au_xino_write(struct super_block *sb, aufs_bindex_t bindex, ino_t h_ino,
 	xi = br->br_xino;
 	file = au_xino_file(xi, calc.idx);
 	if (!file) {
-		/* create and write a new xino file asynchoronously */
+		/* store the inum pair into the list */
+		p = kmalloc(sizeof(*p), GFP_NOFS | __GFP_NOFAIL);
+		p->h_ino = h_ino;
+		p->ino = ino;
+		au_hbl_add(&p->node, &xi->xi_writing);
+
+		/* create and write a new xino file asynchronously */
 		err = au_xino_new_async(sb, br, &calc, ino);
 		if (!err)
 			return 0; /* success */
@@ -1219,16 +1290,15 @@ struct au_xino *au_xino_alloc(unsigned int nfile)
 	spin_lock_init(&xi->xi_nondir.spin);
 	init_waitqueue_head(&xi->xi_nondir.wqh);
 	mutex_init(&xi->xi_mtx);
-	/* init_waitqueue_head(&xi->xi_wq); */
-	/* atomic_set(&xi->xi_pending, 0); */
+	INIT_HLIST_BL_HEAD(&xi->xi_writing);
 	atomic_set(&xi->xi_truncating, 0);
 	kref_init(&xi->xi_kref);
 	goto out; /* success */
 
 out_file:
-	kfree(xi->xi_file);
+	au_kfree_try_rcu(xi->xi_file);
 out_free:
-	kfree(xi);
+	au_kfree_rcu(xi);
 	xi = NULL;
 out:
 	return xi;
@@ -1260,6 +1330,10 @@ static void au_xino_release(struct kref *kref)
 {
 	struct au_xino *xi;
 	int i;
+	unsigned long ul;
+	struct hlist_bl_head *hbl;
+	struct hlist_bl_node *pos, *n;
+	struct au_xi_writing *p;
 
 	xi = container_of(kref, struct au_xino, xi_kref);
 	for (i = 0; i < xi->xi_nfile; i++)
@@ -1268,9 +1342,20 @@ static void au_xino_release(struct kref *kref)
 	for (i = xi->xi_nondir.total - 1; i >= 0; i--)
 		AuDebugOn(xi->xi_nondir.array[i]);
 	mutex_destroy(&xi->xi_mtx);
-	kfree(xi->xi_file);
-	kfree(xi->xi_nondir.array);
-	kfree(xi);
+	hbl = &xi->xi_writing;
+	ul = au_hbl_count(hbl);
+	if (unlikely(ul)) {
+		pr_warn("xi_writing %lu\n", ul);
+		hlist_bl_lock(hbl);
+		hlist_bl_for_each_entry_safe (p, pos, n, hbl, node) {
+			hlist_bl_del(&p->node);
+			au_kfree_rcu(p);
+		}
+		hlist_bl_unlock(hbl);
+	}
+	au_kfree_try_rcu(xi->xi_file);
+	au_kfree_try_rcu(xi->xi_nondir.array);
+	au_kfree_rcu(xi);
 }
 
 int au_xino_put(struct au_branch *br)
@@ -1487,7 +1572,6 @@ void au_xino_clr(struct super_block *sb)
 	xino_clear_xib(sb);
 	xino_clear_br(sb);
 	dbgaufs_brs_del(sb, 0);
-	au_xino_brid_set(sb, -1);
 	sbinfo = au_sbi(sb);
 	/* lvalue, do not call au_mntflags() */
 	au_opt_clr(sbinfo->si_mntflags, XINO);
@@ -1584,8 +1668,6 @@ struct file *au_xino_def(struct super_block *sb)
 			strcat(p, "/" AUFS_XINO_FNAME);
 			AuDbg("%s\n", p);
 			file = au_xino_create(sb, p, /*silent*/0);
-			if (!IS_ERR(file))
-				au_xino_brid_set(sb, br->br_id);
 		}
 		free_page((unsigned long)page);
 	} else {
@@ -1599,8 +1681,6 @@ struct file *au_xino_def(struct super_block *sb)
 			fput(file);
 			file = ERR_PTR(-EINVAL);
 		}
-		if (!IS_ERR(file))
-			au_xino_brid_set(sb, -1);
 	}
 
 out:
